/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software
 * License version 1.1, a copy of which has been included with this
 * distribution in the LICENSE.APL file.  */

package org.apache.log4j.ext;

import java.lang.*;
import java.net.*;

import org.apache.log4j.helpers.*;
import org.opennms.protocols.snmp.*;

/**
 * This class makes use of the JoeSNMP library to implement the underlying SNMP
 * protocol(s).  The JoeSNMP library is a part of the OpenNMS project, and is
 * available/distributed under the GNU Lesser General Public License (LGPL).<p>
 *
 * To quote the JoeSNMP FAQ:<br><pre>
 * "What license is JoeSNMP released under?<br>
 *  <br>
 *	JoeSNMP is released under the GNU Lesser General Public License (LGPL),<br>
 *	documented fully at http://www.fsf.org/copyleft/lesser.html.<br>
 *	Effectively, this means that JoeSNMP is free to distribute and modify as<br>
 *	long as you provide your modified code back to the community.  And if<br>
 *	you'd like to use JoeSNMP as a library within your commercial product,<br>
 *	you are welcome to do so as well, but again, any changes to the<br>
 *	library itself need to be contributed back."</pre><p>
 *
 * You can get more information about OpenNMS <a href="http://www.opennms.org
 * /">here</a>.  You can get a copy of JoeSNMP <a href="http://www.opennms.org
 * /files/releases/joeSNMP/">here</a>.<br>
 *
 * You will need a copy of the library ("joesnmp-0.2.6.jar" at a minimum) to use
 * and/or compile this class.<br>
 * WARNING: The JoeSNMP library <b>requires a minimum of JDK 1.2</b>.
 * <p>
 * @version 2.0.2<br>
 * 2002-10-03<br>
 * changes ---<br>
 *
 * 2001-09-31: mwm : cleaned up the coding style errors, modified the
 * addTrapMessageVariable(String value) to addTrapMessageVariable(String value,
 * String applicationTrapOIDValue) so that the Facade is more flexible.<br>
 *
 * 2001-10-03: mwm : swapped the SNMP library out, replacing the AdventNet
 * commercial library with the one from OpenNMS.  Thanks to <a href="mailto:JZhao@Qcorps.com">
 * Jin Zhao</a> for pointing me towards this!<br>
 *
 * 2001-11-04: mwm : fixed a minor bug with use of the SnmpTrapSession object.  General tidying up.<br>
 *
 * 2002-10-03: mwm : changed the name of the class to "JoeSNMPTrapSender", to reflect the fact that
 * this is now simply the concrete implementation of the new "SnmpTrapSenderFacade" interface that
 * uses JoeSNMP as it's underlying library.  Made changes to deal with the
 * new architecture.<br>
 *
 * 2002-10-15: mwm : fixed a bug that caused some NMS software to receive a the IP address of the
 * sending host incorrectly formatted.<br>
 *
 * 2002-10-15: mwm : changed the sysUpTime value to a long, to cope with the SysUpTimeResolver mechanism.<br>
 *
 * 2002-12-10: mwm : minor tweaks and prettying up of code.<br>
 *
 * 2003-03-21: mwm : fixed a big, nasty, RTFM bug in #sendTrap
 *
 * @author Mark Masterson (<a href="mailto:m.masterson@computer.org">m.masterson@computer.org</a>)<br>
 * <a href="http://www.m2technologies.net/">http://www.m2technologies.net/</a><br>
 */

public class JoeSNMPTrapSender implements SnmpTrapHandler, SnmpTrapSenderFacade {

    private String managementHost = "127.0.0.1";
    private int managementHostTrapListenPort = 162;
    private String enterpriseOID = "1.3.6.1.2.1.1.2.0";
    private String localIPAddress = "127.0.0.1";
    private int localTrapSendPort = 161;
    private int genericTrapType = 0;
    private int specificTrapType = 6;
    private String applicationTrapOID = "1.3.6.1.2.1.1.2.0.0.0.0";
    private String communityString = "public";
    private long sysUpTime = 0;
    private SnmpPduTrap pdu = null;
    private SnmpTrapSession session = null;
    private boolean isInitialized = false;

    /**
     * Default constructor.
     */
    public JoeSNMPTrapSender() {
    }

    /**
     * Skeleton method, implemented only to satisfy the requirements of the
     * JoeSNMP API.  Does nothing except spit out an error message via
     * LogLog.
     */
    public void snmpReceivedTrap(final SnmpTrapSession parm1,
                                 final InetAddress parm2,
                                 final int parm3,
                                 final SnmpOctetString parm4,
                                 final SnmpPduPacket parm5) {
        LogLog.error("This appender does not support receiving traps",
                     new java.lang.UnsupportedOperationException("Method snmpReceivedTrap() not implemented."));
    }

    /**
     * Skeleton method, implemented only to satisfy the requirements of the
     * JoeSNMP API.  Does nothing except spit out an error message via
     * LogLog.
     */
    public void snmpReceivedTrap(final SnmpTrapSession parm1,
                                 final InetAddress parm2,
                                 final int parm3,
                                 final SnmpOctetString parm4,
                                 final SnmpPduTrap parm5) {
        LogLog.error("This appender does not support receiving traps",
                     new java.lang.UnsupportedOperationException("Method snmpReceivedTrap() not implemented."));
    }

    /**
     * Skeleton method, implemented only to satisfy the requirements of the
     * JoeSNMP API.  Does nothing except spit out an error message via
     * LogLog.
     */
    public void snmpTrapSessionError(final SnmpTrapSession parm1,
                                     final int parm2,
                                     final Object parm3) {
        LogLog.error("There was a fatal error at the SNMP session layer.");
    }

    /**
     * This method is called to set the values of all of the class fields used
     * as parameters to the underlying SNMP API.  This method <b>must</b> be
     * called prior to calling either of the other methods in this class.
     *
     * @param managementHostValue - the numeric IP address of the host
     * to which traps should be sent, e.g. "10.255.255.1".  The default is
     * "127.0.0.1", the local loopback address.
     * <p>
     * @param managementHostTrapListenPortValue - any valid TCP/IP port
     * number.  This is the port that the host specified in the
     * "managementHostValue" variable will listen on for SNMP traps. The
     * default is 162.
     * <p>
     * @param enterpriseOIDValue - formatted as an OID
     * E.g. "1.3.6.1.2.1.1.2.0" -- this OID would point to the standard
     * sysObjectID of the "systemName" node of the standard "system" MIB.
     * <br>
     * This is the default value, if none is provided.
     * <br>
     * If you want(need) to use custom OIDs (such as ones from the
     * "private.enterprises" node -- "1.3.6.1.4.1.x.x.x..."), you always need
     * to provide the <b>fully qualified</b> OID as the value of this
     * variable.
     * <p>
     * @param localIPAddressValue - an IP address, as a String, in
     * numeric, dotted decimal format.  E.g. "10.255.255.2". This is the IP
     * address of the host that is using this appender to send SNMP traps.
     * This address will be encoded in the SNMP PDU, and used to provide
     * things like the "agent"'s IP address.
     * <p>
     * @param localTrapSendPortValue - the port that should be used on the
     * local host by the appender to send the trap.  Default is 161.
     * <p>
     * @param genericTrapTypeValue - generic trap type for this SNMP PDU.
     * One of the following values (from the SNMP standard):<br>
     *        0 -- cold start<br>
     *        1 -- warm start<br>
     *        2 -- link down<br>
     *        3 -- link up<br>
     *        4 -- authentification failure<br>
     *        5 -- EGP neighbor loss<br>
     *        6 -- enterprise specific<br>
     * <p>
     * @param specificTrapTypeValue - any value within the range defined
     * for an INTEGER in the ASN.1/BER notation; i.e. -128 to 127.
     * <p>
     * @param communityStringValue - E.g. "public".  This is the
     * default, if none is provided.
     * <p>
     * @param sysUpTimeValue - this is meant to be the amount of time, in
     * seconds, elapsed since the last re-start or re-initialization of the
     * calling application.  Of course, to set this, your application needs
     * to keep track of the value.  The default is 0, if none is provided.
     */
    public void initialize(final String managementHostValue,
                           final int managementHostTrapListenPortValue,
                           final String enterpriseOIDValue,
                           final String localIPAddressValue,
                           final int localTrapSendPortValue,
                           final int genericTrapTypeValue,
                           final int specificTrapTypeValue,
                           final String communityStringValue,
                           final long sysUpTimeValue) {
        this.managementHost = managementHostValue;
        this.managementHostTrapListenPort = managementHostTrapListenPortValue;
        this.enterpriseOID = enterpriseOIDValue;
        this.localIPAddress = localIPAddressValue;
        this.localTrapSendPort = localTrapSendPortValue;
        this.genericTrapType = genericTrapTypeValue;
        this.specificTrapType = specificTrapTypeValue;
        this.communityString = communityStringValue;
        this.sysUpTime = sysUpTimeValue;
        this.pdu = new SnmpPduTrap();
        this.isInitialized = true;
    }

    public void addTrapMessageVariable(final String applicationTrapOIDValue,
                                       final String value) {
        //check pre-condition
        if (!this.isInitialized) {
            LogLog.error("The initialize() method must be called before calling addTrapMessageVariable()");
            return;
        }
        // add OID
        if (applicationTrapOIDValue != null) {
            this.applicationTrapOID = applicationTrapOIDValue;
        }
        final SnmpObjectId oid = new SnmpObjectId(this.applicationTrapOID);
        // set the type
        final SnmpOctetString msg = new SnmpOctetString();
        msg.setString(value);
        // create SnmpVar instance for the value and the type
        try {
            //create varbind
            final SnmpVarBind varbind = new SnmpVarBind(oid, msg);
            // add variable binding
            this.pdu.addVarBind(varbind);
        } catch (Exception e) {
            LogLog.error("Unexpected error creating SNMP bind variable: " + oid + " with value: " + value, e);
        }
    }

    public void sendTrap() {
        //check pre-condition
        if (!this.isInitialized) {
            LogLog.error("The initialize() method must be called before calling sendTrap()");
            return;
        }
        //open the session, set the PDU's values and send the packet
        try {
            this.session = new SnmpTrapSession(this, this.localTrapSendPort);
            final SnmpPeer peer = new SnmpPeer(InetAddress.getByName(this.managementHost));
            peer.setPort(this.managementHostTrapListenPort);
            final SnmpParameters snmpParms = new SnmpParameters();
            snmpParms.setReadCommunity(this.communityString);
            snmpParms.setVersion(SnmpSMI.SNMPV1);
            peer.setParameters(snmpParms);
            if (this.pdu != null) {
                this.pdu.setEnterprise(this.enterpriseOID);
                final SnmpOctetString addr = new SnmpOctetString();
                addr.setString(InetAddress.getByName(this.localIPAddress).getAddress());
                final SnmpIPAddress ipAddr = new SnmpIPAddress(addr);
                this.pdu.setAgentAddress(ipAddr);
                this.pdu.setGeneric(this.genericTrapType);
                this.pdu.setSpecific(this.specificTrapType);
                this.pdu.setTimeStamp(this.sysUpTime);
                if (this.pdu.getLength() > 0) {
                    this.session.send(peer, this.pdu);
                }
            }
        } catch (SnmpPduEncodingException ex) {
            LogLog.error("There were problems with the SNMP parameters -- could not create and send trap", ex);
        } catch (Exception e) {
            LogLog.error("There was an unexpected error", e);
        } finally {
            //this is running on a seperate thread, so make sure it gets
            //cleaned up...
            if (null != this.session && !this.session.isClosed()) this.session.close();
        }
    }
}